; parameters for double correlation
(define-param N 40)
(define-param f 0.5)
(define-param Gamma 0.2)
(define-param h 1)
(define-param deps 0.001)
(define-param randomheight 0.25)
(define-param w1 0)
(define-param w2 0)
(define-param curr 0)


(define-param a 0.5) ; periodicity in microns
(define-param fcen 1.041666667)
(define-param df 1.25)
(define-param nfreq 201)
(define-param textured true)
(define-param deterministic? false)
(define-param no-back-reflector? false)
(define-param siliconthickness 4)
(define-param arthickness 0)
(define-param arindex 1.92)
(define-param randomthickness 0.65)
(define-param gratingthickness 0.0)
(define-param nsilicon 3.679)
(define-param ksilicon 0.00406)
(define-param yperiods 4)
(define-param zperiods 4)
;(define-param nrandom 40)
(define-param reflector-mat-num 0)
  ; 0 = perfect metal; 1=silver; 2=aluminum
(define-param nlayers 0) ; cannot be more than 10
(define-param periodinunitsofa 1)
(define-param boxt 0) ; buried oxide thickness (in units of a)
(define-param boxn 1.46) ; buried oxide refractive index
(define-param is3d? false) ; if false, then run 2D simulation
(set-param! resolution 40)
(define-param rad 0.5)
(define-param sphereeps 1) 
  ; 1 = air spheres (presumably embedded in high dielectric)
(define spheremat (make dielectric (epsilon sphereeps)))
; define different spheres
(define A (make sphere (radius rad) (center 0)
                       (material spheremat)))
(define B (make sphere (radius rad) (center 0 (/ (sqrt 3)))
		       (material spheremat)))
(define C (make sphere (radius rad) (center 0 (/ (sqrt 12)) (/ 2))
		       (material spheremat)))
(define layers (list A B C A B C A B C A))

(define (rev l)
  (if (null? (cdr l))
      l
      (append (rev (cdr l)) (list (car l)))))

(define (choplayers layerlist cnlayers)
  (if (> cnlayers 0)
      (choplayers (rev (cdr (rev layerlist))) (- cnlayers 1))
      layerlist))

(if (> nlayers 0)
  (set! layers (choplayers layers (- 10 nlayers)))
  (set! layers '()))

(print layers)

(define backgroundeps (* nsilicon nsilicon)) ; background for inverse PhC structure

(set! eps-averaging? false)
(define mylambda (/ fcen))
(define pmlthickness 2.0)
(define metalthickness 0.5)
(define decayf 1e-5)

(define pitch (sqrt (/ 2 3)))
(define bufferthickness (/ mylambda 2))
(if (> nlayers 0)
  (define latticeheight (+ (* 2 rad) (* pitch (- nlayers 1))))
  (define latticeheight 0))
(define nx (+ (* 2 (+ pmlthickness bufferthickness)) arthickness siliconthickness metalthickness boxt latticeheight))
(define xsrc (+ (/ nx -2) pmlthickness (/ bufferthickness 2)))
(define xar (+ xsrc (/ (+ bufferthickness arthickness) 2)))
(define xsilicon (+ xar (/ (+ arthickness siliconthickness) 2)))
(define xrandom (- xsilicon (/ siliconthickness 2))) 
(define xmetal (- (/ nx 2) (+ bufferthickness pmlthickness (/ metalthickness 2))))
(define xgrating (- xmetal (/ (+ gratingthickness metalthickness) 2)))
(define xrefl (- xsrc (/ bufferthickness 4)))
(define xtrans (+ xmetal (/ (+ metalthickness bufferthickness) 2)))
(define xphc 0) ; will need to be fixed
(define xbox 0) ; will need to be fixed
(define sigmasilicon (/ (* 4 pi fcen nsilicon ksilicon) (- (* nsilicon nsilicon) (* ksilicon ksilicon))))
(define sy (* periodinunitsofa yperiods))
(if is3d?
  (define sz zperiods)
  (define sz sy))
(define first-layer-depth (- (+ xphc rad) (/ latticeheight 2)))

(if is3d?
  (set! geometry-lattice (make lattice (size nx sy sz)))
  (set! geometry-lattice (make lattice (size nx sy no-size))))
(define ar_m (make medium (index arindex)))
(define silicon_m (make medium (index nsilicon) (D-conductivity sigmasilicon)))

(define-param eps-inf 1.01)
(define-param omegaT0 0.0001)
(define-param omegaP0 (* 6.6793 a))
(define-param g0 (* a 0.0387))
(define-param deps0 (expt (/ omegaP0 omegaT0) 2))

(define-param omegaT1 (* a 0.6581))
(define-param omegaP1 (* a 1.8525))
(define-param g1 (* a 3.1338))
(define-param deps1 (expt (/ omegaP1 omegaT1) 2))

(define-param omegaT2 (* a 3.6137))
(define-param omegaP2 (* a 2.5587))
(define-param g2  (* a 0.3645))
(define-param deps2 (expt (/ omegaP2 omegaT2) 2))

;(define-param omegaT3 (* a 2.8259))
;(define-param omegaP3 (* a 8.5034))
;(define-param g3 (* a 1.2708))
;(define-param deps3 (expt (/ omegaP3 omegaT3) 2))

(define silver (make dielectric (epsilon eps-inf) (polarizations 
                  (make polarizability (omega omegaT0) (gamma g0) (sigma deps0))
                  (make polarizability (omega omegaT1) (gamma g1) (sigma deps1))
                  (make polarizability (omega omegaT2) (gamma g2) (sigma deps2))
                  ;(make polarizability (omega omegaT3) (gamma g3) (sigma deps3))
)))

(define-param omegaT0_alum 0.0001)
(define-param omegaP0_alum (* 8.7366 a))
(define-param g0_alum (* a 0.0379))
(define-param deps0_alum (expt (/ omegaP0_alum omegaT0_alum) 2))

(define-param omegaT1_alum (* a 0.1306))
(define-param omegaP1_alum (* a 5.7558))
(define-param g1_alum (* a 0.2685))
(define-param deps1_alum (expt (/ omegaP1_alum omegaT1_alum) 2))

(define-param omegaT2_alum (* a 1.2452))
(define-param omegaP2_alum (* a 2.7013))
(define-param g2_alum  (* a 0.2516))
(define-param deps2_alum (expt (/ omegaP2_alum omegaT2_alum) 2))

(define-param omegaT3_alum (* a 1.4581))
(define-param omegaP3_alum (* a 4.9220))
(define-param g3_alum (* a 1.0895))
(define-param deps3_alum (expt (/ omegaP3_alum omegaT3_alum) 2))

(define aluminum (make dielectric (epsilon eps-inf) (polarizations 
                  (make polarizability (omega omegaT0_alum) (gamma g0_alum) (sigma deps0_alum))
                  (make polarizability (omega omegaT1_alum) (gamma g1_alum) (sigma deps1_alum))
                  (make polarizability (omega omegaT2_alum) (gamma g2_alum) (sigma deps2_alum))
                  (make polarizability (omega omegaT3_alum) (gamma g3_alum) (sigma deps3_alum))
)))

(if (= reflector-mat-num 0)
  (define backreflmat metal)
  (if (= reflector-mat-num 1)
    (define backreflmat silver)
    (define backreflmat aluminum)))

(if (not deterministic?)
    (let ((time (gettimeofday)))
      (set! *random-state*
            (seed->random-state (+ (car time)
                                   (cdr time))))))

(define-param rand-init? true)

(define-param r 0.1)

; w = f - (f/2)*exp(-Gamma*(N-2*i+2))
; z(n) = w*z(n-1) + (f-w)*z(N-n+2) + h*sqrt(1-f^2)*(random:normal)
; w = f - (f/2)*exp(-Gamma*(N-2*i+1))
; z(N+1-n) = w*z(N+2-n) + (f-w)*z(n) + h*sqrt(1-f^2)*(random:normal)

(define (makerandomblocks currfirst currlast idx)
  (set! w1 (- f (* (/ f 2) (exp (* -1 Gamma (+ (- N (* 2 idx)) 2))))))
  (set! curr (+ (* w1 currfirst) (* (- f w1) currlast) (* h (sqrt(- 1 (* f f))) (random:normal))))
  (set! w2 (- f (* (/ f 2) (exp (* -1 Gamma (+ (- N (* 2 idx)) 1))))))
  (append (list (make block 
                  (center xrandom (- (* (- idx 1) (/ sy N)) (/ sy 2) (/ sy (* 2 N))))
                  (material air)
		  (size (* randomheight (+ currfirst 2)) (+ (/ sy N) deps))))
	  (list (make block 
                  (center xrandom (* -1 (- (* (- idx 1) (/ sy N)) (/ sy 2) (/ sy (* 2 N)))))
                  (material air)
		  (size (* randomheight (+ currlast 2)) (+ (/ sy N) deps))))
	  (if (<= idx (/ N 2))
		(makerandomblocks curr 
				  (+ (* w2 currlast) (* (- f w2) curr) (* h (sqrt(- 1 (* f f))) (random:normal)))
				  (+ idx 1))
		(list ))))

(define currsphere A)
(define shftsphere A)

(define (add-layers mylist height)
  (if (zero? (length mylist))
    (print "Finished running through my layers list.\n")
    (begin
      (set! currsphere (car mylist))
      (set! shftsphere (shift-geometric-object currsphere (vector3 height)))
      (set! geometry (append geometry 
        ; was: (list shftsphere)
        (geometric-objects-duplicates (vector3 0 (/ (sqrt 3) 2) 0.5) -1 1 
	   (geometric-object-duplicates (vector3 0 (/ (sqrt 3) 2) -0.5) -1 1 shftsphere))))
      (add-layers (cdr mylist) (+ height pitch)))))

(if is3d?
  (set! sources (list (make source
                        (src (make gaussian-src (frequency fcen) (fwidth df)))
                        (component Ez) (size 0 sy sz) (center xsrc))))
  (set! sources (list (make source
                        (src (make gaussian-src (frequency fcen) (fwidth df)))
                        (component Ez) (size 0 sy) (center xsrc)))))

(define k-point (vector3 0 0 0))
(set! pml-layers (list (make pml (direction X) (thickness pmlthickness))))

(define trans (add-flux fcen df nfreq
  (make flux-region (center xtrans 0 0) (size 0 sy sz))))
(define refl (add-flux fcen df nfreq
  (make flux-region (center xrefl 0 0) (size 0 sy sz))))
(define refl-flux-file (string-append "refl-flux-" (number->string yperiods) "periods-w" (number->string fcen) "-dw" (number->string df)))
(print "Defined flux regions.\n")

; homogeneous run
(run-until (stop-when-fields-decayed 20 Ez (vector3 xsilicon) decayf))
(display-fluxes trans refl)
(save-flux "refl-flux.h5" refl)
(reset-meep)

(if is3d?
  (set! geometry-lattice (make lattice (size nx sy sz)))
  (set! geometry-lattice (make lattice (size nx sy no-size))))

(define k-point (vector3 0 0 0))
(set! pml-layers (list (make pml (direction X) (thickness pmlthickness))))

(if is3d?
  (set! sources (list (make source
                        (src (make gaussian-src (frequency fcen) (fwidth df)))
                        (component Ez) (size 0 sy sz) (center xsrc))))
  (set! sources (list (make source
                        (src (make gaussian-src (frequency fcen) (fwidth df)))
                        (component Ez) (size 0 sy) (center xsrc)))))

; non-homogeneous run
;set non-homogeneous geometry

(set! geometry (list (make block (center xar) (material ar_m)
                           (size arthickness sy sz))
                        (make block (center xsilicon) (material silicon_m)
                           (size siliconthickness sy sz))))

(if (not no-back-reflector?)
     (set! geometry (append geometry 
                 (list (make block 
                        (center xmetal) (material backreflmat)
		        (size metalthickness sy sz)))
                 (geometric-object-duplicates (vector3 0 periodinunitsofa 0) 0 (- yperiods 1) 
                       (make block (center xgrating (+ 0.5 (/ sy -2)))
                        (material backreflmat)
			(size gratingthickness 0.5 sz))))))

(if textured
    (set! geometry (append geometry
		(list (make block (material silicon_m)
                         (center (+ xrandom 2))
			 (size randomthickness sy sz)))
		(makerandomblocks 0 0 2))))



(if (> boxt 0)
  (set! geometry (append geometry (list (make block (center xbox 0 0)
    (size boxt sy sz) (material (make medium (index boxn))))))))

(if (> nlayers 0)
 (begin
   (set! geometry (append geometry (list (make block (center xphc 0 0) 
         (size latticeheight sy sz)
         (material (make dielectric (epsilon backgroundeps)))))))
  (add-layers layers first-layer-depth)))


(define trans (add-flux fcen df nfreq
  (make flux-region (center xtrans 0 0) (size 0 sy sz))))
(define refl (add-flux fcen df nfreq
  (make flux-region (center xrefl 0 0) (size 0 sy sz))))

(load-minus-flux "refl-flux.h5" refl)

(run-until (stop-when-fields-decayed 20 Ez (vector3 xsilicon) decayf)
  (at-beginning output-epsilon)
  (before-time 400 (at-every 40 output-efield-z)))

(display-fluxes trans refl)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define-param absorberheight 3)
;(define-param dpml 1)
;(define-param sy 10)
;(define-param bufferthickness 3)

;(define nx (+ (* 2 dpml) absorberheight (* 2 bufferthickness) randomheight))
;(define xsrc (- (/ sx 2) (+ dpml (/ bufferthickness 4))))

;(set! geometry (append geometry (list (make block (center (/ absorberheight 2) 0) (material silicon_m) (size absorberheight sy)))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;







